@@ -1,7 +1,12 @@
 """ This module handles TCP connections to the OpenWebNet gateway """
 
 import asyncio
+import hmac
+import hashlib
+import string 
+import random
 import logging
+import traceback
 from urllib.parse import urlparse
 
 from .discovery import find_gateways, get_gateway, get_port
@@ -158,8 +163,23 @@ class OWNSession():
         return await connection.test_connection()
 
     async def test_connection(self) -> dict:
+    
+        retry_count = 0
+        retry_timer = 1
 
-        self._stream_reader, self._stream_writer = await asyncio.open_connection(self._gateway.address, self._gateway.port)
+        while True:
+            try:
+                if retry_count > 2:
+                    self._logger.error("Test session connection still refused after 3 attempts.")
+                    return None
+                self._stream_reader, self._stream_writer = await asyncio.open_connection(self._gateway.address, self._gateway.port)
+                break
+            except ConnectionRefusedError:
+                self._logger.warning("Test session connection refused, retrying in %ss.", retry_timer)
+                await asyncio.sleep(retry_timer)
+                retry_count += 1
+                retry_timer *= 2
+        
         result = await self._negotiate()
         await self.close()
 
@@ -200,10 +220,56 @@ class OWNSession():
             self._logger.debug("Reply: %s", resulting_message)
             self._logger.error("Error while opening %s session.", self._type)
         elif resulting_message.is_SHA():
-            error = True
-            error_message = "unsupported_authentication"
             self._logger.debug("Received SHA challenge: %s", resulting_message)
-            self._logger.error("Error while opening %s session: HMAC authentication not supported.", self._type)
+            if self._gateway.password is None:
+                    error = True
+                    error_message = "password_required"
+                    self._logger.warning("Connection requires a password but none was provided.")
+                    self._stream_writer.write("*#*0##".encode())
+                    await self._stream_writer.drain()
+            else:
+                if resulting_message.is_SHA_1():
+                    self._logger.debug("Detected SHA-1 method.")
+                    method = "sha1"
+                elif resulting_message.is_SHA_256():
+                    self._logger.debug("Detected SHA-256 method.")
+                    method = "sha256"
+                self._logger.debug("Accepting challenge, initiating handshake.")
+                self._stream_writer.write("*#*1##".encode())
+                await self._stream_writer.drain()
+                raw_response = await self._stream_reader.readuntil(OWNSession.SEPARATOR)
+                resulting_message = OWNSignaling(raw_response.decode())
+                if resulting_message.is_nonce():
+                    ra = resulting_message.nonce
+                    self._logger.debug("Received Ra.")
+                    key = ''.join(random.choices(string.digits, k = 56))
+                    rb = self._hex_string_to_int_string(hmac.new(key=key.encode(), digestmod=method).hexdigest())
+                    self._logger.debug("Generated Rb.")
+                    hashedPass = f"*#{rb}*{self._encode_hmac_password(method=method, password=self._gateway.password, nonce_a=ra, nonce_b=rb)}##"
+                    self._logger.debug("Sending %s session password.", self._type)
+                    self._stream_writer.write(hashedPass.encode())
+                    await self._stream_writer.drain() 
+                    try:
+                        raw_response = await self._stream_reader.readuntil(OWNSession.SEPARATOR)
+                        resulting_message = OWNSignaling(raw_response.decode())
+                        if resulting_message.is_nonce():
+                            self._logger.debug("Received HMAC response.")
+                            hmac_response = resulting_message.nonce
+                            if hmac_response == self._decode_hmac_response(method=method, password=self._gateway.password, nonce_a=ra, nonce_b=rb):
+                                self._logger.debug("Server identity confirmed.")
+                                self._stream_writer.write("*#*1##".encode())
+                                await self._stream_writer.drain()
+                            else:
+                                self._logger.error("Server identity could not be confirmed.")
+                                self._stream_writer.write("*#*0##".encode())
+                                await self._stream_writer.drain()
+                                error = True
+                                error_message = "negociation_error"
+                                self._logger.error("Error while opening %s session: HMAC authentication failed.", self._type)
+                    except asyncio.IncompleteReadError:
+                        error = True
+                        error_message = "password_error"
+                        self._logger.error("Password error while opening %s session.", self._type)
         elif resulting_message.is_nonce():
             self._logger.debug("Received nonce: %s", resulting_message)
             if self._gateway.password is None:
@@ -232,10 +298,14 @@ class OWNSession():
         elif resulting_message.is_ACK():
             self._logger.debug("Reply: %s", resulting_message)
             self._logger.info("%s session established.", self._type.capitalize())
+        else:
+            error = True
+            error_message = "negotiation_failed"
+            self._logger.info("Unexpected message during negotiation: %s", resulting_message)
 
         return {"Success": not error, "Message": error_message}
 
-    def _get_own_password (self, password, nonce, test=False) :
+    def _get_own_password(self, password, nonce, test=False):
         start = True    
         num1 = 0
         num2 = 0
@@ -287,6 +357,38 @@ class OWNSession():
             num2 = num1
         return num1
 
+    def _encode_hmac_password(self, method: str, password: str, nonce_a: str, nonce_b: str):
+        if method == 'sha1':
+            message = self._int_string_to_hex_string(nonce_a) + self._int_string_to_hex_string(nonce_b) + "736F70653E" + "636F70653E" + hashlib.sha1(password.encode()).hexdigest()
+            return self._hex_string_to_int_string(hashlib.sha1(message.encode()).hexdigest())
+        elif method == 'sha256':
+            message = self._int_string_to_hex_string(nonce_a) + self._int_string_to_hex_string(nonce_b) + "736F70653E" + "636F70653E" + hashlib.sha256(password.encode()).hexdigest()
+            return self._hex_string_to_int_string(hashlib.sha256(message.encode()).hexdigest())
+        else:
+            return None
+
+    def _decode_hmac_response(self, method: str, password: str, nonce_a: str, nonce_b: str):
+        if method == 'sha1':
+            message = self._int_string_to_hex_string(nonce_a) + self._int_string_to_hex_string(nonce_b) + hashlib.sha1(password.encode()).hexdigest()
+            return self._hex_string_to_int_string(hashlib.sha1(message.encode()).hexdigest())
+        elif method == 'sha256':
+            message = self._int_string_to_hex_string(nonce_a) + self._int_string_to_hex_string(nonce_b) + hashlib.sha256(password.encode()).hexdigest()
+            return self._hex_string_to_int_string(hashlib.sha256(message.encode()).hexdigest())
+        else:
+            return None
+
+    def _int_string_to_hex_string(self, int_string: str) -> str:
+        hex_string = ""
+        for i in range(0, len(int_string), 2):
+            hex_string += f"{int(int_string[i:i+2]):x}"
+        return hex_string
+
+    def _hex_string_to_int_string(self, hex_string: str) -> str:
+        int_string = ""
+        for i in range(0, len(hex_string), 1):
+            int_string += f"{int(hex_string[i:i+1], 16):0>2d}"
+        return int_string
+
 class OWNEventSession(OWNSession):
 
     def __init__(self, gateway: OWNGateway = None, logger: logging.Logger = None):
@@ -299,7 +401,23 @@ class OWNEventSession(OWNSession):
 
     async def connect(self):
         self._logger.info("Opening event session.")
-        self._stream_reader, self._stream_writer = await asyncio.open_connection(self._gateway.address, self._gateway.port)
+        
+        retry_count = 0
+        retry_timer = 1
+
+        while True:
+            try:
+                if retry_count > 4:
+                    self._logger.error("Event session connection still refused after 5 attempts.")
+                    return None
+                self._stream_reader, self._stream_writer = await asyncio.open_connection(self._gateway.address, self._gateway.port)
+                break
+            except ConnectionRefusedError:
+                self._logger.warning("Event session connection refused, retrying in %ss.", retry_timer)
+                await asyncio.sleep(retry_timer)
+                retry_count += 1
+                retry_timer *= 2
+
         await self._negotiate()
     
     async def get_next(self):
@@ -308,17 +426,22 @@ class OWNEventSession(OWNSession):
         try:
             data = await self._stream_reader.readuntil(OWNSession.SEPARATOR)
             return OWNMessage.parse(data.decode())
-        except asyncio.IncompleteReadError:
-            self._logger.warning("Incomplete read on the event bus: %s.", data.decode())
+        except asyncio.IncompleteReadError as ire:
+            self._logger.warning("Connection interrupted, reconnecting...")
+            #traceback.print_tb(ire.__traceback__)
+            await self.connect()
             return None
-        except AttributeError:
-            self._logger.error("Received data could not be parsed into a message: %s", data.decode())
+        except AttributeError as ar:
+            self._logger.exception("Received data could not be parsed into a message:")
+            #traceback.print_tb(ar.__traceback__)
             return None
         except ConnectionError as e:
-            self._logger.error("Connection error: %s", e)
+            self._logger.exception("Connection error:")
+            #traceback.print_tb(e.__traceback__)
             return None
         except Exception as e:
-            self._logger.error("Error: \"%s\" while parsing %s", e, data)
+            self._logger.exception("Error:")
+            #traceback.print_tb(e.__traceback__)
             return None
 
     async def close(self):
@@ -343,9 +466,23 @@ class OWNCommandSession(OWNSession):
 
         self._logger.info("Opening command session.")
 
-        self._stream_reader, self._stream_writer = await asyncio.open_connection(self._gateway.address, self._gateway.port)
-        message_string = str(message).encode()
-        negotiation_result = await self._negotiate()
+        retry_count = 0
+        retry_timer = 0.5
+
+        while True:
+            try:
+                if retry_count > 3:
+                    self._logger.error("Command session connection still refused after 4 attempts.")
+                    return None
+                self._stream_reader, self._stream_writer = await asyncio.open_connection(self._gateway.address, self._gateway.port)
+                message_string = str(message).encode()
+                negotiation_result = await self._negotiate()
+                break
+            except (ConnectionRefusedError, asyncio.IncompleteReadError):
+                self._logger.warning("Command session connection refused or negotiation interrupted, retrying in %ss.", retry_timer)
+                await asyncio.sleep(retry_timer)
+                retry_count += 1
+                retry_timer *= 2
         
         if negotiation_result["Success"]:
             self._stream_writer.write(message_string)
