@@ -11,11 +11,24 @@ MESSAGE_TYPE_DAILY_CONSUMPTION = "daily_consumption"
 MESSAGE_TYPE_MONTHLY_CONSUMPTION = "monthly_consumption"
 MESSAGE_TYPE_CURRENT_DAY_CONSUMPTION = "current_day_partial_consumption"
 MESSAGE_TYPE_CURRENT_MONTH_CONSUMPTION = "current_month_partial_consumption"
+MESSAGE_TYPE_MAIN_TEMPERATURE = "main_temperature"
+MESSAGE_TYPE_SECONDARY_TEMPERATURE = "secondary_temperature"
+MESSAGE_TYPE_TARGET_TEMPERATURE = "target_temperature"
+MESSAGE_TYPE_LOCAL_OFFSET = "local_offset"
+MESSAGE_TYPE_LOCAL_TARGET_TEMPERATURE = "local_targer_temperature"
+MESSAGE_TYPE_MODE = "hvac_mode"
+MESSAGE_TYPE_MODE_TARGET = "hvac_mode_target"
+MESSAGE_TYPE_ACTION = "hvac_action"
+
+CLIMATE_MODE_OFF = "off"
+CLIMATE_MODE_HEAT = "heat"
+CLIMATE_MODE_COOL = "cool"
+CLIMATE_MODE_AUTO = "auto"
 
 class OWNMessage():
 
     _ACK = re.compile(r"^\*#\*1##$") #  *#*1##
-    _NACK = re.compile(r"^\*#\*0##$") #  *#*1##
+    _NACK = re.compile(r"^\*#\*0##$") #  *#*0##
     _COMMAND_SESSION = re.compile(r"^\*99\*0##$") #  *99*0##
     _EVENT_SESSION = re.compile(r"^\*99\*1##$") #  *99*1##
     _NONCE = re.compile(r"^\*#(\d+)##$") #  *#123456789##
@@ -34,8 +47,10 @@ class OWNMessage():
         self._family = ""
         self._who = ""
         self._where = ""
+        self._is_valid_message = False
 
         if self._STATUS.match(self._raw):
+            self._is_valid_message = True
             self._match = self._STATUS.match(self._raw)
             self._family = 'EVENT'
             self._type = 'STATUS'
@@ -53,6 +68,7 @@ class OWNMessage():
             self._dimension_value = None
 
         elif self._STATUS_REQUEST.match(self._raw):
+            self._is_valid_message = True
             self._match = self._STATUS_REQUEST.match(self._raw)
             self._family = 'REQUEST'
             self._type = 'STATUS_REQUEST'
@@ -67,6 +83,7 @@ class OWNMessage():
             self._dimension_value = None
         
         elif self._DIMENSION_REQUEST.match(self._raw):
+            self._is_valid_message = True
             self._match = self._DIMENSION_REQUEST.match(self._raw)
             self._family = 'REQUEST'
             self._type = 'DIMENSION_REQUEST'
@@ -81,6 +98,7 @@ class OWNMessage():
             self._dimension_value = None
 
         elif self._DIMENSION_REQUEST_REPLY.match(self._raw):
+            self._is_valid_message = True
             self._match = self._DIMENSION_REQUEST_REPLY.match(self._raw)
             self._family = 'EVENT'
             self._type = 'DIMENSION_REQUEST_REPLY'
@@ -97,6 +115,7 @@ class OWNMessage():
             del self._dimension_value[0]
 
         elif self._DIMENSION_WRITING.match(self._raw):
+            self._is_valid_message = True
             self._match = self._DIMENSION_WRITING.match(self._raw)
             self._family = 'COMMAND'
             self._type = 'DIMENSION_WRITING'
@@ -131,20 +150,28 @@ class OWNMessage():
         else:
             return None
 
+    @property
     def is_event(self) -> bool:
         return self._family == 'EVENT'
 
+    @property
     def is_command(self) -> bool:
         return self._family == 'COMMAND'
 
+    @property
     def is_request(self) -> bool:
         return self._family == 'REQUEST'
 
+    @property
     def is_translation(self) -> bool:
         return self._family == 'COMMAND_TRANSLATION'
 
     @property
-    def who(self) -> str:
+    def is_valid(self) -> bool:
+        return self._is_valid_message
+
+    @property
+    def who(self) -> int:
         """ The 'who' ID of the subject of this message """
         return self._who
 
@@ -168,6 +195,53 @@ class OWNMessage():
         """ A human readable log of the event """
         return self._human_readable_log
 
+    @property
+    def is_general(self) -> bool:
+        if self.who == 1 or self.who == 2:
+            if self._where == '0':
+                return True
+            else:
+                return False
+        else:
+            return False
+
+    @property
+    def is_group(self) -> bool:
+        if self.who == 1 or self.who == 2:
+            if self._where.startswith('#'):
+                return True
+            else:
+                return False
+        else:
+            return False
+
+    @property
+    def is_area(self) -> bool:
+        if self.who == 1 or self.who == 2:
+            try:
+                if (self._where == '00' or self._where == '100' or (len(self._where) == 1 and int(self._where) > 0 and int(self._where) < 10)):
+                    return True
+                else:
+                    return False
+            except ValueError:
+                return False
+        else:
+            return False
+
+    @property
+    def group(self) ->  int:
+        if self.is_group:
+            return int(self._where[1:])
+        else:
+            return None
+
+    @property
+    def area(self) -> int:
+        if self.is_area:
+            return 10 if self._where == '100' else int(self._where)
+        else:
+            return None
+
     def __repr__(self) -> str:
         return self._raw
 
@@ -383,6 +457,250 @@ class OWNHeatingEvent(OWNEvent):
     def __init__(self, data):
         super().__init__(data)
 
+        self._type = None
+
+        self._zone = int(self._where[1:]) if self._where.startswith('#') else int(self._where)
+        if self._zone == 0 and self._where_param is not None:
+            self._zone = int(self._where_param[0])
+        self._sensor = None
+        if self._zone > 99:
+            self._sensor = int(str(self._zone)[:1])
+            self._zone = int(str(self._zone)[1:])
+        self._actuator = None
+        
+        self._mode = None
+        self._mode_name = None
+        self._set_temperature = None
+        self._local_offset = None
+        self._local_set_temperature = None
+        self._measured_temperature = None
+        self._secondary_temperature = None
+
+        self._is_active = None
+        self._is_heating = None
+        self._is_cooling = None
+
+        self._fan_on = None
+        self._fan_speed = None
+        self._cooling_fan_on = None
+        self._cooling_fan_speed = None
+
+        _valve_active_states = ["1", "2", "6", "7", "8"]
+        _actuator_active_states = ["1", "2", "6", "7", "8", "9"]
+
+        if self._what is not None:
+            self._mode = int(self._what)
+            if self._mode in [103,203,303,102,202,302]:
+                self._type = MESSAGE_TYPE_MODE
+                self._mode_name = CLIMATE_MODE_OFF
+                self._human_readable_log = f"Zone {self._zone}'s mode is set to '{self._mode_name}'"
+            elif self._mode in [0,210,211,215] or (self._mode >= 2101 and self._mode <= 2103) or (self._mode >= 2201 and self._mode <= 2216):
+                self._type = MESSAGE_TYPE_MODE
+                self._mode_name = CLIMATE_MODE_COOL
+                self._human_readable_log = f"Zone {self._zone}'s mode is set to '{self._mode_name}'"
+            elif self._mode in [1,110,111,115] or (self._mode >= 1101 and self._mode <= 1103) or (self._mode >= 1201 and self._mode <= 1216):
+                self._type = MESSAGE_TYPE_MODE
+                self._mode_name = CLIMATE_MODE_HEAT
+                self._human_readable_log = f"Zone {self._zone}'s mode is set to '{self._mode_name}'"
+            elif self._mode in [310,311,315] or (self._mode >= 23001 and self._mode <= 23255) or (self._mode >= 13001 and self._mode <= 13255):
+                self._type = MESSAGE_TYPE_MODE
+                self._mode_name = CLIMATE_MODE_AUTO
+                self._human_readable_log = f"Zone {self._zone}'s mode is set to '{self._mode_name}'"
+            elif self._mode == 20:
+                self._mode_name = None
+                self._human_readable_log = f"Zone {self._zone}'s remote control is disabled"
+            elif self._mode == 21:
+                self._mode_name = None
+                self._human_readable_log = f"Zone {self._zone}'s remote control is enabled"
+            else:
+                self._mode_name = None
+                self._human_readable_log = f"Zone {self._zone}'s mode is unknown"
+                
+            if self._what_param and self._what_param[0] is not None:
+                self._type = MESSAGE_TYPE_MODE_TARGET
+                self._set_temperature = float(f"{self._what_param[0][1:3]}.{self._what_param[0][-1]}")
+                self._human_readable_log += f" at {self._set_temperature}°C."
+            else:
+                self._human_readable_log += "."
+
+        if self._dimension == 0:    #Temperature
+            if self._sensor is None:
+                self._type = MESSAGE_TYPE_MAIN_TEMPERATURE
+                self._measured_temperature = float(f"{self._dimension_value[0][1:3]}.{self._dimension_value[0][-1]}")
+                self._human_readable_log = f"Zone {self._zone}'s main sensor is reporting a temperature of {self._measured_temperature}°C."
+            else:
+                self._type = MESSAGE_TYPE_SECONDARY_TEMPERATURE
+                self._secondary_temperature = float(f"{self._dimension_value[0][1:3]}.{self._dimension_value[0][-1]}")
+                self._human_readable_log = f"Zone {self._zone}'s secondary sensor {self._sensor} is reporting a temperature of {self._secondary_temperature}°C."
+
+        elif self._dimension == 11: #Fan speed
+            _fan_mode = int(self._dimension_value[0])
+            if _fan_mode < 4:
+                self._fan_on = True
+                self._is_active = True
+                if _fan_mode > 0:
+                    self._fan_speed = _fan_mode
+                    self._human_readable_log = f"Zone {self._zone}'s fan is on at speed {self._fan_speed}."
+                else:
+                    self._human_readable_log = f"Zone {self._zone}'s fan is on at 'Auto' speed."
+            else:
+                self._fan_on = False
+                self._is_active = False
+                self._human_readable_log = f"Zone {self._zone}'s fan is off."
+
+        elif self._dimension == 12: #Local set temperature (set+offset)
+            self._type = MESSAGE_TYPE_LOCAL_TARGET_TEMPERATURE
+            self._local_set_temperature = float(f"{self._dimension_value[0][1:3]}.{self._dimension_value[0][-1]}")
+            self._human_readable_log = f"Zone {self._zone}'s local target temperature is set to {self._local_set_temperature}°C."
+
+        elif self._dimension == 13: #Local offset
+            self._type = MESSAGE_TYPE_LOCAL_OFFSET
+            if (self._dimension_value[0] == "0" or self._dimension_value[0] == "00" or self._dimension_value[0] == "4" or self._dimension_value[0] == "5"):
+                self._local_offset = 0
+            elif self._dimension_value[0].startswith('0'):
+                self._local_offset = int(f"+{self._dimension_value[0][1:]}")
+            else:
+                self._local_offset = int(f"-{self._dimension_value[0][1:]}")
+            self._human_readable_log = f"Zone {self._zone}'s local offset is set to {self._local_offset}°C."
+
+        elif self._dimension == 14: #Set temperature
+            self._type = MESSAGE_TYPE_TARGET_TEMPERATURE
+            self._set_temperature = float(f"{self._dimension_value[0][1:3]}.{self._dimension_value[0][-1]}")
+            self._human_readable_log = f"Zone {self._zone}'s target temperature is set to {self._set_temperature}°C."
+
+        elif self._dimension == 19: #Valves status
+            self._type = MESSAGE_TYPE_ACTION
+            self._is_cooling = self._dimension_value[0] in _valve_active_states
+            self._is_heating = self._dimension_value[1] in _valve_active_states
+            self._is_active = self._is_cooling | self._is_heating
+            # Handle cooling valve status relative to fan speed/status
+            _cooling_value = int(self._dimension_value[0])
+            if _cooling_value == 0:
+                self._human_readable_log = f"Zone {self._zone}'s cooling valve is off"
+            elif _cooling_value == 1:
+                self._human_readable_log = f"Zone {self._zone}'s cooling valve is on"
+            elif _cooling_value == 2:
+                self._human_readable_log = f"Zone {self._zone}'s cooling valve is opened"
+            elif _cooling_value == 3:
+                self._human_readable_log = f"Zone {self._zone}'s cooling valve is closed"
+            elif _cooling_value == 4:
+                self._human_readable_log = f"Zone {self._zone}'s cooling valve is stopped"
+            elif _cooling_value > 4:
+                _fan_mode = _cooling_value - 5
+                if _fan_mode > 0:
+                    self._cooling_fan_on = True
+                    self._is_active = True
+                    self._cooling_fan_speed = _fan_mode
+                    self._human_readable_log = f"Zone {self._zone}'s cooling fan is on at speed {self._fan_speed}"
+                else:
+                    self._cooling_fan_on = False
+                    self._is_active = False
+                    self._human_readable_log = f"Zone {self._zone}'s cooling fan is off"
+            # Handle heating valve status relative to fan speed/status
+            _heating_value = int(self._dimension_value[1])
+            if _heating_value == 0:
+                self._human_readable_log += "; heating valve is off."
+            elif _heating_value == 1:
+                self._human_readable_log += "; heating valve is on."
+            elif _heating_value == 2:
+                self._human_readable_log += "; heating valve is opened."
+            elif _heating_value == 3:
+                self._human_readable_log += "; heating valve is closed."
+            elif _heating_value == 4:
+                self._human_readable_log += "; heating valve is stopped."
+            elif _heating_value > 4:
+                _fan_mode = _heating_value - 5
+                if _fan_mode > 0:
+                    self._fan_on = True
+                    self._is_active = True
+                    self._fan_speed = _fan_mode
+                    self._human_readable_log += f"; heating fan is on at speed {self._fan_speed}."
+                else:
+                    self._fan_on = False
+                    self._is_active = False
+                    self._human_readable_log += "; heating fan is off."
+
+        elif self._dimension == 20: #Actuator status
+            self._type = MESSAGE_TYPE_ACTION
+            self._is_active = self._dimension_value[0] in _actuator_active_states
+            self._actuator = self._where_param[0] if self._where_param[0] is not None else 1
+            _value = int(self._dimension_value[0])
+            if _value == 0:
+                self._human_readable_log = f"Zone {self._zone}'s actuator {self._actuator} is off."
+            elif _value == 1:
+                self._human_readable_log = f"Zone {self._zone}'s actuator {self._actuator} is on."
+            elif _value == 2:
+                self._human_readable_log = f"Zone {self._zone}'s actuator {self._actuator} is opened."
+            elif _value == 3:
+                self._human_readable_log = f"Zone {self._zone}'s actuator {self._actuator} is closed."
+            elif _value == 4:
+                self._human_readable_log = f"Zone {self._zone}'s actuator {self._actuator} is stopped."
+            elif _value > 4:
+                _fan_mode = _value - 5
+                if _fan_mode > 0:
+                    self._fan_on = True
+                    self._is_active = True
+                    if _fan_mode < 4:
+                        self._fan_speed = _fan_mode
+                        self._human_readable_log = f"Zone {self._zone}'s fan is on at speed {self._fan_speed}."
+                    else:
+                        self._human_readable_log = f"Zone {self._zone}'s fan is on at 'Auto' speed."
+                else:
+                    self._fan_on = False
+                    self._is_active = False
+                    self._human_readable_log = f"Zone {self._zone}'s fan is off."
+
+    @property
+    def unique_id(self) -> str:
+        """ The ID of the subject of this message """
+        if self._zone == 0:
+            return f"{self._who}-#0"
+        elif self._sensor is not None:
+            return f"{self._who}-{self._where}"
+        else:
+            return f"{self._who}-{self._zone}"
+
+    @property
+    def message_type(self):
+        return self._type
+
+    @property
+    def zone(self) -> int:
+        return self._zone
+
+    @property
+    def mode(self) -> str:
+        return self._mode_name
+
+    def is_active(self) -> bool:
+        return self._is_active
+
+    def is_heating(self) -> bool:
+        return self._is_heating
+
+    def is_cooling(self) -> bool:
+        return self._is_cooling
+
+    @property
+    def main_temperature(self) -> float:
+        return self._measured_temperature
+
+    @property
+    def secondary_temperature(self):
+        return [self._sensor, self._secondary_temperature]
+    
+    @property
+    def set_temperature(self) -> float:
+        return self._set_temperature
+    
+    @property
+    def local_offset(self) -> int:
+        return self._local_offset
+    
+    @property
+    def local_set_temperature(self) -> float:
+        return self._local_set_temperature
+
 class OWNAlarmEvent(OWNEvent):
 
     def __init__(self, data):
@@ -737,38 +1055,43 @@ class OWNEnergyEvent(OWNEvent):
                 self._human_readable_log = f"Sensor {self._sensor} is reporting an active power draw of {self._active_power} W."
             elif self._dimension == 511:
                 _now = datetime.date.today()
-                _messageDate =  datetime.date(_now.year, int(self._dimension_param[0]), int(self._dimension_param[1]))
-                if _messageDate > _now:
-                    _messageDate.replace(year= _now.year - 1)
+                _raw_message_date = datetime.date(_now.year, int(self._dimension_param[0]), int(self._dimension_param[1]))
+                try:
+                    if _raw_message_date > _now:
+                        _message_date = datetime.date(_now.year - 1, int(self._dimension_param[0]), int(self._dimension_param[1]))
+                    else:
+                        _message_date = datetime.date(_now.year, int(self._dimension_param[0]), int(self._dimension_param[1]))
+                except ValueError as e:
+                    return None
 
                 if int(self._dimension_value[0]) != 25:
                     self._type = MESSAGE_TYPE_HOURLY_CONSUMPTION
-                    self._hourly_consumption['date'] = _messageDate
-                    self._hourly_consumption['hour'] = int(self._dimension_value[0])
+                    self._hourly_consumption['date'] = _message_date
+                    self._hourly_consumption['hour'] = int(self._dimension_value[0])-1
                     self._hourly_consumption['value'] = int(self._dimension_value[1])
                     self._human_readable_log = f"Sensor {self._sensor} is reporting a power consumtion of {self._hourly_consumption['value']} Wh for {self._hourly_consumption['date']} at {self._hourly_consumption['hour']}."
                 else:
                     self._type = MESSAGE_TYPE_DAILY_CONSUMPTION
-                    self._daily_consumption['date'] = _messageDate
+                    self._daily_consumption['date'] = _message_date
                     self._daily_consumption['value'] = int(self._dimension_value[1])
                     self._human_readable_log = f"Sensor {self._sensor} is reporting a power consumtion of {self._daily_consumption['value']} Wh for {self._daily_consumption['date']}."
             elif self._dimension == 513 or self._dimension == 514:
                 _now = datetime.date.today()
-                _messageMonth = datetime.date(_now.year, int(self._dimension_param[0]), 1)
+                _raw_message_date = datetime.date(_now.year, int(self._dimension_param[0]), 1)
                 try:
-                    if self._dimension == 513 and _messageMonth > _now:
-                        _messageDate = datetime.date(_now.year - 1, int(self._dimension_param[0]), int(self._dimension_value[0]))
+                    if self._dimension == 513 and _raw_message_date > _now:
+                        _message_date = datetime.date(_now.year - 1, int(self._dimension_param[0]), int(self._dimension_value[0]))
                     elif self._dimension == 514:
-                        if _messageMonth > _now:
-                            _messageDate = datetime.date(_now.year - 2, int(self._dimension_param[0]), int(self._dimension_value[0]))
+                        if _raw_message_date > _now:
+                            _message_date = datetime.date(_now.year - 2, int(self._dimension_param[0]), int(self._dimension_value[0]))
                         else:
-                            _messageDate = datetime.date(_now.year - 1, int(self._dimension_param[0]), int(self._dimension_value[0]))
+                            _message_date = datetime.date(_now.year - 1, int(self._dimension_param[0]), int(self._dimension_value[0]))
                     else:
-                        _messageDate = datetime.date(_now.year, int(self._dimension_param[0]), int(self._dimension_value[0]))
+                        _message_date = datetime.date(_now.year, int(self._dimension_param[0]), int(self._dimension_value[0]))
                 except ValueError as e:
                     return None
                 self._type = MESSAGE_TYPE_DAILY_CONSUMPTION
-                self._daily_consumption['date'] = _messageDate
+                self._daily_consumption['date'] = _message_date
                 self._daily_consumption['value'] = int(self._dimension_value[1])
                 self._human_readable_log = f"Sensor {self._sensor} is reporting a power consumtion of {self._daily_consumption['value']} Wh for {self._daily_consumption['date']}."
             elif self._dimension == 54:
@@ -777,8 +1100,8 @@ class OWNEnergyEvent(OWNEvent):
                 self._human_readable_log = f"Sensor {self._sensor} is reporting a power consumtion of {self._current_day_partial_consumption} Wh up to now today."
             elif self._dimension == 52:
                 self._type = MESSAGE_TYPE_MONTHLY_CONSUMPTION
-                _messageDate =  datetime.date(int(f"20{self._dimension_param[0]}"), self._dimension_param[1], 1)
-                self._monthly_consumption['date'] = _messageDate
+                _message_date =  datetime.date(int(f"20{self._dimension_param[0]}"), self._dimension_param[1], 1)
+                self._monthly_consumption['date'] = _message_date
                 self._monthly_consumption['value'] = int(self._dimension_value[0])
                 self._human_readable_log = f"Sensor {self._sensor} is reporting a power consumtion of {self._monthly_consumption['value']} Wh for {self._monthly_consumption['date'].strftime('%B %Y')}."
             elif self._dimension == 53:
@@ -927,7 +1250,7 @@ class OWNCommand(OWNMessage):
             elif _who == 2:
                 return OWNAutomationCommand(data)
             elif _who == 4:
-                return cls(data)
+                return OWNHeatingCommand(data)
             elif _who == 5:
                 return cls(data)
             elif _who == 9:
@@ -1014,6 +1337,79 @@ class OWNAutomationCommand(OWNCommand):
         message._human_readable_log = f"Setting shutter {where} position to {level}%."
         return message
 
+class OWNHeatingCommand(OWNCommand):
+
+    def __init__(self, data):
+        super().__init__(data)
+
+    @classmethod
+    def status(cls, where):
+        message = cls(f"*#4*{where}##")
+        message._human_readable_log = f"Requesting climate status update for {where}."
+        return message
+
+    @classmethod
+    def get_temperature(cls, where):
+        message = cls(f"*#4*{where}*0##")
+        message._human_readable_log = f"Requesting climate status update for {where}."
+        return message
+
+    @classmethod
+    def set_mode(cls, where, mode: str, standalone=False):
+        zone = int(where.split("#")[-1]) if where.startswith('#') else int(where)
+        zone_name = f"zone {zone}" if zone > 0 else "general"
+
+        if standalone:
+            zone = f"#{zone}" if zone == 0 else str(zone)
+        else:
+            zone = f"#{zone}"
+
+        mode_name = mode
+        if mode == CLIMATE_MODE_OFF:
+            mode = 303
+        elif mode == CLIMATE_MODE_AUTO:
+            mode = 311
+        else:
+            return None
+
+        message = cls(f"*4*{mode}*{zone}##")
+        message._human_readable_log = f"Setting {zone_name} mode to '{mode_name}'."
+        return message
+
+    @classmethod
+    def turn_off(cls, where, standalone=False):
+        return cls.set_mode(where=where, mode=CLIMATE_MODE_OFF, standalone=standalone)
+
+    @classmethod
+    def set_temperature(cls, where, temperature: float, mode: str, standalone=False):
+        zone = int(where.split("#")[-1]) if where.startswith('#') else int(where)
+        zone_name = f"zone {zone}" if zone > 0 else "general"
+
+        if standalone:
+            zone = f"#{zone}" if zone == 0 else str(zone)
+        else:
+            zone = f"#{zone}"
+
+        temperature = round(temperature * 2) / 2
+        if temperature < 5.0:
+            temperature = 5.0
+        elif temperature > 40.0:
+            temperature = 40.0
+        temperature_print = f"{temperature}"
+        temperature = int(temperature * 10)
+
+        mode_name = mode
+        if mode == CLIMATE_MODE_HEAT:
+            mode = 1
+        elif mode == CLIMATE_MODE_COOL:
+            mode = 2
+        elif mode == CLIMATE_MODE_AUTO:
+            mode = 3
+
+        message = cls(f"*#4*{zone}*#14*{temperature:04d}*{mode}##")
+        message._human_readable_log = f"Setting {zone_name} to {temperature_print}°C in mode '{mode_name}'."
+        return message
+
 class OWNGatewayCommand(OWNCommand):
 
     def __init__(self, data):
@@ -1136,8 +1532,6 @@ class OWNEnergyCommand(OWNCommand):
         message._human_readable_log = f"Requesting monthly power consumption for {year}-{month} from sensor {where}."
         return message
 
-
-
 class OWNDryContactCommand(OWNCommand):
 
     def __init__(self, data):
@@ -1154,6 +1548,9 @@ class OWNSignaling(OWNMessage):
 
     def __init__(self, data):
         self._raw = data
+        self._family = None
+        self._type = 'UNKNOWN'
+        self._human_readable_log = data
 
         if self._ACK.match(self._raw):
             self._match = self._ACK.match(self._raw)
@@ -1173,8 +1570,8 @@ class OWNSignaling(OWNMessage):
         elif self._SHA.match(self._raw):
             self._match = self._SHA.match(self._raw)
             self._family = 'SIGNALING'
-            self._type = 'SHA'
-            self._human_readable_log = f"SHA{'-1' if self._match.group(1) else '-256'} challenge received."
+            self._type = f"SHA{'-1' if self._match.group(1) == '1' else '-256'}"
+            self._human_readable_log = f"SHA{'-1' if self._match.group(1) == '1' else '-256'} challenge received."
         elif self._COMMAND_SESSION.match(self._raw):
             self._match = self._COMMAND_SESSION.match(self._raw)
             self._family = 'SIGNALING'
@@ -1212,4 +1609,10 @@ class OWNSignaling(OWNMessage):
         return self._type == 'NONCE'
 
     def is_SHA(self) -> bool:
-        return self._type == 'SHA'
\ No newline at end of file
+        return self._type == 'SHA-1' or self._type == 'SHA-256'
+
+    def is_SHA_1(self) -> bool:
+        return self._type == 'SHA-1'
+
+    def is_SHA_256(self) -> bool:
+        return self._type == 'SHA-256'
